from __future__ import unicode_literals
import json
import uuid
from datetime import timedelta
from django.conf import settings
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.db import models, IntegrityError
from django.db.models import QuerySet
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _


def _get_current_datetime():
    return timezone.now()


def get_time_to_live_minutes(app):
    """
    Get the configured time to live in minutes for tokens for the given ``app``.
    """
    time_to_live_minutes = getattr(
        settings,
        'DJANGO_CRADMIN_SECURE_USER_TOKEN_TIME_TO_LIVE_MINUTES', {
            'default': 60*24*4  # Default to 4 days
        })
    return time_to_live_minutes.get(app, time_to_live_minutes['default'])


def get_expiration_datetime_for_app(app):
    """
    Get the expiration datetime of tokens for the given ``app``
    relative to ``now``.

    If the given app is configured to with 60 minutes time to live,
    this will return a datetime object representing 60 minutes in the future.
    """
    time_to_live_minutes = get_time_to_live_minutes(app)
    return _get_current_datetime() + timedelta(minutes=time_to_live_minutes)


def generate_token():
    """
    Generate a token for the :obj:`.GenericTokenWithMetadata.token` field.

    Joins an UUID1 (unique uuid) with an UUID4 (random uuid), so the chance
    of this not beeing unique is very low, and guessing this is
    very hard.

    Returns:
        A token that is very unlikely to not be unique.
    """
    return u'{}-{}'.format(uuid.uuid1(), uuid.uuid4())


class GenericTokenExpiredError(Exception):
    """
    Raised by :meth:`.GenericTokenWithMetadata.get_and_validate` when
    the token is found, but has expired.
    """


class GenericTokenWithMetadataQuerySet(QuerySet):
    """
    QuerySet for :class:`.GenericTokenWithMetadata`.
    """
    def unsafe_pop(self, app, token):
        """
        Get the :class:`.GenericTokenWithMetadata` matching the given
        `token` and `app`. Removes the GenericTokenWithMetadata from the database, and
        returns the `GenericTokenWithMetadata` object.

        You should normally use :meth:`.GenericTokenWithMetadataBaseManager.pop`
        instead of this.

        Raises:
            GenericTokenWithMetadata.DoesNotExist if no matching token is stored for
            the given app.
        """
        token = self.get(token=token, app=app)
        token.delete()
        return token

    def filter_has_expired(self):
        """
        Return a queryset containing only the expired GenericTokenWithMetadata objects in
        the current queryset.
        """
        return self.filter(
            expiration_datetime__isnull=False,
            expiration_datetime__lt=_get_current_datetime())

    def filter_not_expired(self):
        """
        Return a queryset containing only the un-expired GenericTokenWithMetadata objects in
        the current queryset.
        """
        return self.filter(
            models.Q(expiration_datetime=None) |
            models.Q(expiration_datetime__gte=_get_current_datetime()))

    def filter_by_content_object(self, content_object):
        """
        Filter by :obj:`.GenericTokenWithMetadata.content_object`.

        Examples:
            Lets say the content_object is a User object, you
            can find all tokens for that user in the
            ``page_admin_invites`` app like this::

                from django.contrib.auth import get_user_model
                user = get_user_model()
                GenericTokenWithMetadata.objects\
                    .filter(app='page_admin_invites')\
                    .filter_by_content_object(user)
        """
        content_type = ContentType.objects.get_for_model(content_object)
        return self.filter(
            object_id=content_object.pk,
            content_type=content_type)

    def filter_usable_by_content_object_in_app(self, content_object, app):
        """
        Filters only non-expired tokens with the given ``content_object`` and ``app``.
        """
        return self.filter_not_expired()\
            .filter(app=app)\
            .filter_by_content_object(content_object)


class GenericTokenWithMetadataBaseManager(models.Manager):
    """
    Manager for :class:`.GenericTokenWithMetadata`.

    Inherits all methods from :class:`.GenericTokenWithMetadataQuerySet`.
    """
    def generate(self, app, expiration_datetime, content_object, metadata=None):
        """
        Generate and save a token for the given user and app.

        Returns:
            A :class:`.GenericTokenWithMetadata` object with a token
            that is guaranteed to be unique.
        """
        token = generate_token()
        generic_token_with_metadata = GenericTokenWithMetadata(
            content_object=content_object, app=app, token=token,
            created_datetime=_get_current_datetime(),
            expiration_datetime=expiration_datetime)
        if metadata:
            generic_token_with_metadata.metadata = metadata

        try:
            generic_token_with_metadata.full_clean()
        except ValidationError as e:
            if 'token' in e.error_dict and e.error_dict['token'][0].code == 'unique':
                return self.generate(
                    app=app, content_object=content_object,
                    expiration_datetime=expiration_datetime, metadata=metadata)
            else:
                raise

        try:
            generic_token_with_metadata.save()
        except IntegrityError:
            return self.generate(
                app=app, content_object=content_object,
                expiration_datetime=expiration_datetime, metadata=metadata)
        else:
            return generic_token_with_metadata

    def pop(self, app, token):
        """
        Get the :class:`.GenericTokenWithMetadata` matching the given
        `token` and `app`. Removes the GenericTokenWithMetadata from the database, and
        returns the `GenericTokenWithMetadata` object.

        Does not return expired tokens.

        Raises:
            GenericTokenWithMetadata.DoesNotExist: If no matching token is stored for the
                given app, or if the token is expired.
        """
        return self.filter_not_expired().unsafe_pop(app=app, token=token)

    def delete_expired(self):
        """
        Delete all expired tokens.
        """
        self.filter_has_expired().delete()

    def get_and_validate(self, app, token):
        """
        Get the given ``token`` for the given ``app``.

        Raises:
            GenericTokenWithMetadata.DoesNotExist: If the token does not exist.
            GenericTokenExpiredError: If the token has expired.
        """
        token = GenericTokenWithMetadata.objects.get(app=app, token=token)
        if token.is_expired():
            raise GenericTokenExpiredError('The token has expired.')
        return token


class GenericTokenWithMetadata(models.Model):
    """
    Provides a secure token with attached metadata suitable for
    email and sharing workflows like password reset, public share urls, etc.
    """
    objects = GenericTokenWithMetadataBaseManager.from_queryset(GenericTokenWithMetadataQuerySet)()

    #: The app that generated the token.
    #: You should set this to the name of the app the
    #: generated the token.
    app = models.CharField(db_index=True, max_length=255)

    #: A unique and random token, set it using :func:`.generate_token`.
    token = models.CharField(max_length=100, unique=True)

    #: Datetime when the token was created.
    created_datetime = models.DateTimeField(
        verbose_name=_('Created'))

    #: Datetime when the token expires.
    #: This can be `None`, which means that the token does not expire.
    expiration_datetime = models.DateTimeField(
        null=True, default=None, blank=True,
        verbose_name=_('Expires'))

    #: Single use?
    #: If this is `False`, the token can be used an unlimited number of
    #: times.
    single_use = models.BooleanField(default=True)

    #: JSON encoded metadata
    metadata_json = models.TextField(null=False, blank=True, default='')

    #: The content-type of the :obj:`~.GenericTokenWithMetadata.content_object`.
    #: Together with :obj:`~.GenericTokenWithMetadata.object_id` this creates a
    #: generic foreign key to any Django model.
    content_type = models.ForeignKey(ContentType)

    #: The object ID of the :obj:`~.GenericTokenWithMetadata.content_object`.
    #: Together with :obj:`~.GenericTokenWithMetadata.content_type` this creates a
    #: generic foreign key to any Django model.
    object_id = models.PositiveIntegerField()

    #: A :class:`django.contrib.contenttypes.fields.GenericForeignKey` to
    #: the object this token is for.
    #:
    #: This generic relationship is used to associate the token
    #: with a specific object.
    #:
    #: Use cases:
    #:
    #: - Password reset: Use the content_object to link to a User object when you
    #:   create password reset tokens.
    #: - Invites: Use the content_object to link to the object that you
    #:   are inviting users to. This enables you to filter on the
    #:   content object to show pending shares.
    content_object = GenericForeignKey('content_type', 'object_id')

    def is_expired(self):
        """
        Returns `True` if :obj:`.GenericTokenWithMetadata.expiration_datetime` is in the past,
        and `False` if it is in the future or now.
        """
        return self.expiration_datetime is not None and self.expiration_datetime < _get_current_datetime()

    @property
    def metadata(self):
        """
        Decode :obj:`.GenericTokenWithMetadata.metadata_json` and return the result.

        Return `None` if metadata_json is empty.
        """
        if self.metadata_json:
            if not hasattr(self, '_metadata'):
                # Store the decoded metadata to avoid re-decoding the json for
                # each access. We invalidate this cache in the setter.
                self._metadata = json.loads(self.metadata_json)
            return self._metadata
        else:
            return None

    @metadata.setter
    def metadata(self, metadata):
        """
        Set :obj:`.GenericTokenWithMetadata.metadata_json`. Encodes the given
        metadata using `json.dumps`.
        """
        self.metadata_json = json.dumps(metadata)
        if hasattr(self, '_metadata'):
            delattr(self, '_metadata')



from __future__ import unicode_literals
import json
from xml.sax.saxutils import quoteattr
from django import forms
from django.core.urlresolvers import reverse
from django.template.loader import render_to_string


class BulkFileUploadWidget(forms.Widget):
    template_name = 'django_cradmin/apps/cradmin_temporaryfileuploadstore/bulkfileupload-widget.django.html'

    def __init__(self,
                 accept=None,
                 apiparameters=None,
                 dropbox_text=None,
                 invalid_filetype_message=None,
                 advanced_fileselectbutton_text=None,
                 simple_fileselectbutton_text=None):
        """
        Parameters:
            accept (str): Comma separated string of filetypes that we should accept.
                Added to the file upload field as the accept attribute.
                Note that this is not validated serverside - use the accept attribute
                of ``apiparameters`` to do that.
            apiparameters (dict): Dict of API parameters. Here you can
                put values for the ``minutes_to_live``, ``accept``, ``max_filename_length``
                and ``unique_filenames`` attributes of
                :class:`django_cradmin.apps.cradmin_temporaryfileuploadstore.models.TemporaryFileCollection`.

                Example::

                    apiparameters = {
                        'minutes_to_live': 10,
                        'accept': 'application/pdf,text/plain,image/*',
                        'unique_filenames': True
                    }
        """
        self.accept = accept
        self.apiparameters = apiparameters or {}
        self.dropbox_text = dropbox_text
        self.invalid_filetype_message = invalid_filetype_message
        self.advanced_fileselectbutton_text = advanced_fileselectbutton_text
        self.simple_fileselectbutton_text = simple_fileselectbutton_text
        super(BulkFileUploadWidget, self).__init__(attrs=None)

    def get_apiurl(self):
        return reverse('cradmin_temporary_file_upload_api')

    def get_template_context_data(self, **context):
        """
        Can be overridden to adjust the template context data.
        """
        context['accept'] = self.accept
        context['dropbox_text'] = self.dropbox_text
        context['invalid_filetype_message'] = self.invalid_filetype_message
        context['advanced_fileselectbutton_text'] = self.advanced_fileselectbutton_text
        context['simple_fileselectbutton_text'] = self.simple_fileselectbutton_text
        context['apiparameters'] = quoteattr(json.dumps(self.apiparameters))
        context['apiurl'] = self.get_apiurl()
        return context

    def render(self, name, value, attrs=None):
        if value is None:
            value = ''
        return render_to_string(self.template_name, self.get_template_context_data(
            hiddenfieldname=name,
            fieldvalue=value))



from django.http import HttpResponse
from django.views.generic import View

from django_cradmin.apps.cradmin_email import emailutils


class DemoEmail(emailutils.AbstractEmail):
    subject_template = 'cradmin_email/cradmin_email_send_testmail/subject.django.txt'
    html_message_template = 'cradmin_email/cradmin_email_send_testmail/html_message.django.html'

    def get_context_data(self):
        context = super().get_context_data()
        context['name'] = 'Test Name'
        return context


class EmailDesignView(View):
    def get(self, request, format='html'):
        email = DemoEmail()
        if format == 'plaintext':
            return HttpResponse(email.render_plaintext_message(), content_type='text/plain')
        else:
            return HttpResponse(email.render_html_message())



from __future__ import unicode_literals
from datetime import timedelta, datetime

from django.contrib.auth.models import AnonymousUser
from django.core.exceptions import PermissionDenied
from django.http import HttpResponse

from django.test import TestCase, RequestFactory
from django.utils import timezone
import htmls
import mock

from django_cradmin.apps.cradmin_generic_token_with_metadata.models import GenericTokenWithMetadata
from django_cradmin.apps.cradmin_invite.baseviews.accept import AbstractAcceptInviteView
from django_cradmin.tests.helpers import create_user


class AcceptInviteView(AbstractAcceptInviteView):
    def get_appname(self):
        return 'testapp'

    def invite_accepted(self, token):
        return HttpResponse('OK')


class TestAbstractAcceptInviteView(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        # request._messages = mock.MagicMock()

    def __create_token(self, metadata=None, expiration_datetime=None, **kwargs):
        generic_token_with_metadata = GenericTokenWithMetadata(
            created_datetime=timezone.now(),
            expiration_datetime=(expiration_datetime or (timezone.now() + timedelta(days=2))),
            content_object=create_user('invitecontentobject'),
            app='testapp',
            **kwargs)
        generic_token_with_metadata.metadata = metadata or {}
        generic_token_with_metadata.save()
        return generic_token_with_metadata

    def test_get_expired_token(self):
        self.__create_token(
            token='valid-token',
            expiration_datetime=datetime(2014, 1, 1))
        request = self.factory.get('/test')
        response = AcceptInviteView.as_view()(request, token='valid-token')
        selector = htmls.S(response.content)
        self.assertTrue(selector.exists('#django_cradmin_invite_expired_message'))
        self.assertEqual(
            selector.one('#django_cradmin_invite_expired_message').alltext_normalized,
            'This invite link has expired.')

    def test_get_invalid_token(self):
        self.__create_token(
            token='valid-token')
        request = self.factory.get('/test')
        response = AcceptInviteView.as_view()(request, token='invalid-token')
        selector = htmls.S(response.content)
        self.assertTrue(selector.exists('#django_cradmin_invite_invalid_token_message'))
        self.assertEqual(
            selector.one('#django_cradmin_invite_invalid_token_message').alltext_normalized,
            'Invalid invite URL. Are you sure you copied the entire URL from the email?')

    def test_get_render(self):
        request = self.factory.get('/test')
        request.user = mock.MagicMock()
        token = self.__create_token()
        response = AcceptInviteView.as_view()(request, token=token.token)
        self.assertEqual(response.status_code, 200)
        response.render()
        selector = htmls.S(response.content)
        self.assertEqual(selector.one('title').alltext_normalized, 'Accept invite')
        self.assertEqual(selector.one('.page-header h1').alltext_normalized, 'Accept invite')
        self.assertEqual(
            selector.one('#django_cradmin_invite_accept_description').alltext_normalized,
            'TODO: Set a description_template for your AbstractAcceptInviteView subclass.')

    def test_get_render_is_authenticated(self):
        request = self.factory.get('/test')
        request.user = create_user('testuser')
        token = self.__create_token()
        with self.settings(DJANGO_CRADMIN_SITENAME='Testsite'):
            response = AcceptInviteView.as_view()(request, token=token.token)
        self.assertEqual(response.status_code, 200)
        response.render()
        selector = htmls.S(response.content)
        self.assertTrue(selector.exists('button#django_cradmin_invite_accept_as_button'))
        self.assertEqual(
            selector.one('#django_cradmin_invite_accept_register_account_button')['href'],
            '/cradmin_register_account/begin?next=http%3A%2F%2Ftestserver%2Ftest')
        self.assertEqual(
            selector.one('#django_cradmin_invite_accept_login_as_different_user_button')['href'],
            '/cradmin_authenticate/logout?'
            'next=%2Faccounts%2Flogin%2F%3Fnext%3Dhttp%253A%252F%252Ftestserver%252Ftest')

        self.assertEqual(
            selector.one('button#django_cradmin_invite_accept_as_button').alltext_normalized,
            'Accept as testuser')
        self.assertEqual(
            selector.one('#django_cradmin_invite_accept_register_account_button').alltext_normalized,
            'Sign up for Testsite')
        self.assertEqual(
            selector.one('#django_cradmin_invite_accept_login_as_different_user_button').alltext_normalized,
            'Sign in as another user')

    def test_get_render_not_authenticated(self):
        request = self.factory.get('/test')
        request.user = AnonymousUser()
        token = self.__create_token()
        with self.settings(DJANGO_CRADMIN_SITENAME='Testsite'):
            response = AcceptInviteView.as_view()(request, token=token.token)
        self.assertEqual(response.status_code, 200)
        response.render()
        selector = htmls.S(response.content)
        self.assertEqual(
            selector.one('#django_cradmin_invite_accept_login_button')['href'],
            '/accounts/login/?next=http%3A%2F%2Ftestserver%2Ftest')
        self.assertEqual(
            selector.one('#django_cradmin_invite_accept_register_account_button')['href'],
            '/cradmin_register_account/begin?next=http%3A%2F%2Ftestserver%2Ftest')

        self.assertEqual(
            selector.one('#django_cradmin_invite_accept_login_button').alltext_normalized,
            'Sign in')
        self.assertEqual(
            selector.one('#django_cradmin_invite_accept_register_account_button').alltext_normalized,
            'Sign up for Testsite')

    def test_post_not_authenticated(self):
        request = self.factory.post('/test')
        request.user = AnonymousUser()
        token = self.__create_token()
        with self.assertRaises(PermissionDenied):
            AcceptInviteView.as_view()(request, token=token.token)

    def test_post_is_authenticated(self):
        request = self.factory.post('/test')
        request.user = create_user('testuser')
        token = self.__create_token()
        response = AcceptInviteView.as_view()(request, token=token.token)
        self.assertEqual(response.content, b'OK')



from __future__ import unicode_literals
from django.contrib.auth.models import User
from django.core.exceptions import ValidationError
from django.test import TestCase
from django_cradmin.apps.cradmin_register_account.forms.auth_user import AuthUserCreateAccountForm, \
    AuthUserCreateAccountAutoUsernameForm
from django_cradmin.tests.helpers import create_user


class TestAuthUserCreateAccountForm(TestCase):
    def test_is_valid(self):
        form = AuthUserCreateAccountForm({
            'username': 'test',
            'password1': 'test',
            'password2': 'test',
            'email': 'test@example.com'
        })
        self.assertTrue(form.is_valid())

    def test_save(self):
        form = AuthUserCreateAccountForm({
            'username': 'test',
            'password1': 'test',
            'password2': 'test',
            'email': 'test@example.com'
        })
        form.is_valid()
        self.assertEquals(User.objects.count(), 0)
        form.save()
        self.assertEquals(User.objects.count(), 1)
        created_user = User.objects.first()
        self.assertEqual(created_user.username, 'test')
        self.assertEqual(created_user.email, 'test@example.com')
        self.assertTrue(created_user.has_usable_password())
        self.assertTrue(created_user.check_password('test'))

    def test_passwords_do_not_match(self):
        form = AuthUserCreateAccountForm({
            'username': 'unused',
            'password1': 'test1',
            'password2': 'test2',
            'email': 'unused@example.com'
        })
        self.assertFalse(form.is_valid())
        self.assertEquals(form.non_field_errors(), [u'The passwords do not match.'])

    def test_email_is_not_unique(self):
        create_user('testuser', email='test@example.com')
        form = AuthUserCreateAccountForm({
            'username': 'unused',
            'password1': 'unused',
            'password2': 'unused',
            'email': 'test@example.com'
        })
        self.assertFalse(form.is_valid())
        self.assertEquals(form.errors, {
            'email': [u'Account with this email address already exists.']
        })

    def test_username_is_not_unique(self):
        create_user('test')
        form = AuthUserCreateAccountForm({
            'username': 'test',
            'password1': 'unused',
            'password2': 'unused',
            'email': 'unused@example.com'
        })
        self.assertFalse(form.is_valid())
        self.assertEquals(form.errors, {
            'username': [u'User with this Username already exists.']
        })


class TestAuthUserCreateAccountAutoUsernameForm(TestCase):
    def test_is_valid(self):
        form = AuthUserCreateAccountAutoUsernameForm({
            'password1': 'test',
            'password2': 'test',
            'email': 'test@example.com'
        })
        self.assertTrue(form.is_valid())

    def test_save(self):
        form = AuthUserCreateAccountAutoUsernameForm({
            'password1': 'test',
            'password2': 'test',
            'email': 'test@example.com'
        })
        form.is_valid()
        self.assertEquals(User.objects.count(), 0)
        form.save()
        self.assertEquals(User.objects.count(), 1)
        created_user = User.objects.first()
        self.assertEqual(created_user.username, 'test@example.com')
        self.assertEqual(created_user.email, 'test@example.com')
        self.assertTrue(created_user.has_usable_password())
        self.assertTrue(created_user.check_password('test'))

    def test_passwords_do_not_match(self):
        form = AuthUserCreateAccountAutoUsernameForm({
            'password1': 'test1',
            'password2': 'test2',
            'email': 'unused@example.com'
        })
        self.assertFalse(form.is_valid())
        self.assertEquals(form.non_field_errors(), [u'The passwords do not match.'])

    def test_email_is_not_unique(self):
        create_user('testuser', email='test@example.com')
        form = AuthUserCreateAccountAutoUsernameForm({
            'password1': 'unused',
            'password2': 'unused',
            'email': 'test@example.com'
        })
        self.assertFalse(form.is_valid())
        self.assertEquals(form.errors, {
            'email': [u'Account with this email address already exists.']
        })

    def test_email_is_more_than_30_chars(self):
        create_user('testuser', email='test@example.com')
        form = AuthUserCreateAccountAutoUsernameForm({
            'password1': 'unused',
            'password2': 'unused',
            'email': 'a.very.long.testuser@example.com'
        })
        self.assertTrue(form.is_valid())
        user = form.save()
        self.assertEquals(user.username, 'a.very.long.testuser@example.c')

    def test_email_makes_username_not_unique(self):
        create_user(
            username='a.very.long.testuser@example.c',
            email='a.very.long.testuser@example.co.uk')
        form = AuthUserCreateAccountAutoUsernameForm({
            'password1': 'unused',
            'password2': 'unused',
            'email': 'a.very.long.testuser@example.com'
        })
        self.assertTrue(form.is_valid())
        with self.assertRaisesMessage(ValidationError, 'User with this Username already exists.'):
            form.save()



from __future__ import unicode_literals
from django_cradmin.crispylayouts import CradminSubmitButton


class BulkFileUploadSubmit(CradminSubmitButton):
    template = 'django_cradmin/apps/cradmin_temporaryfileuploadstore/bulkfileupload-submit.django.html'
    extra_button_attributes = {
        'django-cradmin-bulkfileupload-submit': ''
    }

    def __init__(self, name, value, uploading_text=None, uploading_icon_cssclass=None, **kwargs):
        self.uploading_text = uploading_text or value
        self.uploading_icon_cssclass = uploading_icon_cssclass
        super(BulkFileUploadSubmit, self).__init__(
            name, value, **kwargs)



from __future__ import unicode_literals
import json
from django import forms
from django.template.loader import render_to_string


class AceMarkdownWidget(forms.widgets.Textarea):
    template_name = 'django_cradmin/acemarkdown/widget.django.html'
    directiveconfig = {
        # 'showTextarea': False,
        # 'theme': 'tomorrow'
    }

    @property
    def media(self):
        return forms.Media(
            js=[
                'django_cradmin/dist/vendor/js/ace-editor/ace.js',
            ]
        )

    def render(self, name, value, attrs=None):
        attrs = attrs.copy()
        attrs['textarea django-cradmin-acemarkdown-textarea'] = ''
        textarea = super(AceMarkdownWidget, self).render(name, value, attrs)
        return render_to_string(
            self.template_name, {
                'textarea': textarea,
                'directiveconfig': json.dumps(self.directiveconfig)
            }
        )



from django.forms import widgets
from django.forms.utils import flatatt
from django.template import loader
from django.utils.translation import ugettext_lazy as _


class DatePickerWidget(widgets.DateInput):
    """
    A Widget using AngularJS ui-bootstrap's `ui.bootstrap.datepicker` for selecting a date
    """
    template_name = 'django_cradmin/widgets/datepicker.django.html'

    def __init__(self, attrs=None, format=None, placeholder=_('yyyy-mm-dd')):
        if attrs is None:
            attrs = {}
        if placeholder:
            attrs['placeholder'] = placeholder
        super(DatePickerWidget, self).__init__(attrs=attrs, format=format)

    def render(self, name, value, attrs=None):
        if value is None:
            value = ''
        final_attrs = self.build_attrs(attrs, type=self.input_type, name=name)
        return loader.render_to_string(self.template_name, {
            'dateformat': 'yyyy-MM-dd',
            'datevalue': value,
            'fieldid': attrs.get('id', 'id_{}'.format(name)),
            'fieldname': name,
            'show_date': True,
            'attrs': flatatt(final_attrs)
        })


class TimePickerWidget(widgets.TimeInput):
    """
    Time picker widget that is wrapped in a div with the
    ``django-cradmin-timepicker`` css class.
    """
    template_name = 'django_cradmin/widgets/timepicker.django.html'

    def __init__(self, attrs=None, format=None, placeholder=_('hh:mm')):
        if attrs is None:
            attrs = {}
        if placeholder:
            attrs['placeholder'] = placeholder
        super(TimePickerWidget, self).__init__(attrs=attrs, format=format)

    def render(self, name, value, attrs=None):
        inputfield = super(TimePickerWidget, self).render(name, value, attrs)
        return loader.render_to_string(self.template_name, {
            'inputfield': inputfield,
            'fieldname': name
        })


class DateTimePickerWidget(widgets.MultiWidget):
    """
    A Widget using AngularJS ui-bootstrap's `ui.bootstrap.datepicker` for selecting a date and
    `ui.bootstrap.timepicker` for selecting a time, then combining them in a single datetime
    """

    def __init__(self, attrs=None,
                 datewidget_placeholder=_('yyyy-mm-dd'),
                 timewidget_placeholder=_('hh:mm')):
        _widgets = [
            DatePickerWidget(attrs=attrs, placeholder=datewidget_placeholder),
            TimePickerWidget(attrs=attrs, placeholder=timewidget_placeholder)
        ]
        super(DateTimePickerWidget, self).__init__(_widgets, attrs)

    def decompress(self, value):
        return [value, value]

    def format_output(self, rendered_widgets):
        return u'<div class="django-cradmin-datetimepicker">{}</div>'.format(
            u''.join(rendered_widgets))

    def value_from_datadict(self, data, files, name):
        timevalue = data.get('{}_1'.format(name), '')
        datevalue = data.get('{}_0'.format(name), '')
        datetimevalue = '{} {}'.format(datevalue, timevalue)
        return datetimevalue



from __future__ import unicode_literals
from django.core.urlresolvers import reverse
from django.test import TestCase
import htmls


class TestEmailSentView(TestCase):
    def setUp(self):
        self.url = reverse('cradmin-resetpassword-email-sent')

    def test_get(self):
        with self.settings(DJANGO_CRADMIN_SITENAME='Testsite'):
            response = self.client.get(self.url)
        selector = htmls.S(response.content)
        self.assertEqual(selector.one('h1').alltext_normalized, 'Check your email')
        self.assertIn(
            'We have sent you an email. Click the link in the email to reset your password.',
            selector.one('#django_cradmin_focusedlayout_content').alltext_normalized)
        self.assertIn(
            'If you do not see the email, check your junk folder or try searching for "Reset your Testsite password".',
            selector.one('#django_cradmin_focusedlayout_content').alltext_normalized)



from __future__ import unicode_literals
from builtins import object
from django import http
from django.contrib import messages
from django.utils.translation import ugettext_lazy as _
from crispy_forms import layout
from django import forms
from django_cradmin.apps.cradmin_temporaryfileuploadstore.crispylayouts import BulkFileUploadSubmit
from django_cradmin.apps.cradmin_temporaryfileuploadstore.models import TemporaryFileCollection
from django_cradmin.apps.cradmin_temporaryfileuploadstore.widgets import BulkFileUploadWidget
from django_cradmin.templatetags.cradmin_icon_tags import cradmin_icon

from django_cradmin.viewhelpers import objecttable
from django_cradmin.viewhelpers import create
from django_cradmin.viewhelpers import update
from django_cradmin.viewhelpers import delete
from django_cradmin.viewhelpers import formbase
from django_cradmin import crapp
from django_cradmin.apps.cradmin_imagearchive.models import ArchiveImage
from django_cradmin.widgets import filewidgets


class DescriptionColumn(objecttable.MultiActionColumn):
    modelfield = 'description'

    def render_value(self, obj):
        return obj.screenreader_text

    def get_buttons(self, obj):
        return [
            objecttable.Button(
                label=_('Edit'),
                url=self.reverse_appurl('edit', args=[obj.id])),
            objecttable.Button(
                label=_('Delete'),
                url=self.reverse_appurl('delete', args=[obj.id]),
                buttonclass="btn btn-danger btn-sm"),
        ]


class DescriptionSelectColumn(objecttable.UseThisActionColumn):
    modelfield = 'description'

    def render_value(self, obj):
        return obj.screenreader_text

    def get_buttons(self, obj):
        return [
            objecttable.UseThisButton(
                view=self.view,
                label=_('Use this'),
                obj=obj)
        ]


class ImageColumn(objecttable.ImagePreviewColumn):
    modelfield = 'image'
    preview_width = 100
    preview_height = 65
    column_width = '100px'

    def get_header(self):
        return _('Preview')


class ArchiveImagesQuerySetForRoleMixin(object):
    """
    Used by listing, update and delete view to ensure
    that only images that the current role has access to
    is available.
    """
    def get_queryset_for_role(self, role):
        return ArchiveImage.objects.filter_owned_by_role(role)\
            .order_by('-created_datetime')


class ArchiveImagesListView(ArchiveImagesQuerySetForRoleMixin, objecttable.ObjectTableView):
    model = ArchiveImage
    columns = [
        ImageColumn,
        DescriptionColumn,
    ]
    searchfields = ['name', 'description', 'file_extension']
    hide_column_headers = True

    def get_buttons(self):
        app = self.request.cradmin_app
        return [
            objecttable.Button(label=_('Add image'),
                               url=app.reverse_appurl('create'),
                               buttonclass='btn btn-primary'),
            objecttable.Button(label=_('Bulk upload'),
                               url=app.reverse_appurl('bulkadd'),
                               buttonclass='btn btn-default'),
        ]


class ArchiveImagesSingleSelectView(ArchiveImagesQuerySetForRoleMixin, objecttable.ObjectTableView):
    model = ArchiveImage
    columns = [
        ImageColumn,
        DescriptionSelectColumn,
    ]
    searchfields = ['name', 'description', 'file_extension']
    hide_menu = True
    # paginate_by = 10
    hide_column_headers = True

    def make_foreignkey_preview_for(self, obj):
        return obj.get_preview_html()

    def get_buttons(self):
        app = self.request.cradmin_app
        return [
            objecttable.ForeignKeySelectButton(
                label=_('Add image'),
                buttonclass='btn btn-primary',
                request=self.request,
                url=app.reverse_appurl('create')),
        ]


class ArchiveImageCreateUpdateMixin(object):
    model = ArchiveImage
    roleid_field = 'role'

    def get_form(self, form_class=None):
        form = super(ArchiveImageCreateUpdateMixin, self).get_form(form_class=form_class)
        form.fields['image'].widget = filewidgets.ImageWidget(
            preview_width=300, preview_height=300, clearable=False)
        return form


class ArchiveImageCreateView(ArchiveImageCreateUpdateMixin, create.CreateView):
    """
    View used to create new images.
    """
    fields = ['image', 'description']

    submit_use_label = _('Upload and select')
    submit_save_label = _('Upload image')
    submit_save_and_continue_edititing_label = _('Upload and continue editing')

    def get_field_layout(self):
        return [
            layout.Div('image', css_class="cradmin-focusfield"),
            layout.Div(
                layout.Field('description', css_class='cradmin-textarea-small'),
                css_class="cradmin-focusfield"),
        ]

    def save_object(self, form, commit=True):
        archiveimage = super(ArchiveImageCreateView, self).save_object(form, commit=False)
        image = archiveimage.image
        archiveimage.image = None
        archiveimage.save()
        archiveimage.image = image
        archiveimage.save()
        return archiveimage

    def get_success_message(self, object):
        what = u'"{}"'.format(object)
        return _(u'Uploaded %(what)s.') % {'what': what}


class ArchiveImageUpdateView(ArchiveImagesQuerySetForRoleMixin, ArchiveImageCreateUpdateMixin, update.UpdateView):
    """
    View used to create edit existing images.
    """
    fields = ['image', 'description']

    def get_field_layout(self):
        return [
            layout.Div('image', css_class="cradmin-focusfield"),
            layout.Div(
                layout.Field('description', css_class='cradmin-textarea-small'),
                css_class="cradmin-focusfield"),
        ]


class ArchiveImageDeleteView(ArchiveImagesQuerySetForRoleMixin, delete.DeleteView):
    """
    View used to delete existing images.
    """


class BulkAddForm(forms.Form):
    filecollectionid = forms.IntegerField(
        required=True,
        widget=BulkFileUploadWidget(
            accept='image/*',
            # accept='image/png,image/jpeg,image/gif',  # NOTE: Does not work with the fileselector in firefox
            apiparameters={
                'accept': 'image/png,image/jpeg,image/gif'
            },
            dropbox_text=_('Upload images by dragging and dropping them here'),
            invalid_filetype_message=_('Invalid filetype. You can only upload images.'),
            advanced_fileselectbutton_text=_('... or select images'),
            simple_fileselectbutton_text=_('Select images ...')
        ),
        label=_('Upload at least one image'),
        help_text=_(
            'Upload as many images as you like. '
            'You can edit the name and description of the images after they have been uploaded.'),
        error_messages={
            'required': _('You must upload at least one file.')
        })


class ArchiveImageBulkAddView(formbase.FormView):
    template_name = 'django_cradmin/apps/cradmin_imagearchive/bulkadd.django.html'
    form_class = BulkAddForm
    form_attributes = {
        'django-cradmin-bulkfileupload-form': ''
    }
    form_id = 'django_cradmin_imagearchive_bulkadd_form'
    extra_form_css_classes = ['django-cradmin-form-noasterisk']

    def get_buttons(self):
        return [
            BulkFileUploadSubmit(
                'submit', _('Add to the image archive'),
                uploading_text=_('Uploading images'),
                uploading_icon_cssclass=cradmin_icon('loadspinner')),
        ]

    def get_field_layout(self):
        return [
            layout.Div(
                'filecollectionid',
                # css_class="cradmin-globalfields"),
                css_class="cradmin-focusfield"),
        ]

    def get_formhelper(self):
        formhelper = super(ArchiveImageBulkAddView, self).get_formhelper()
        formhelper.form_show_labels = False
        return formhelper

    def upload_file_to_archive(self, temporaryfile):
        archiveimage = ArchiveImage(
            role=self.request.cradmin_role,
            name=temporaryfile.filename)
        archiveimage.clean()
        archiveimage.save()
        archiveimage.image.save(temporaryfile.filename, temporaryfile.file)
        archiveimage.full_clean()

    def upload_files_to_archive(self, temporaryfilecollection):
        for temporaryfile in temporaryfilecollection.files.all():
            self.upload_file_to_archive(temporaryfile)

    def get_collectionqueryset(self):
        return TemporaryFileCollection.objects\
            .filter_for_user(self.request.user)\
            .prefetch_related('files')

    def get_success_message(self, temporaryfilecollection):
        """
        Override this to provide a success message.

        The ``temporaryfilecollection`` is the TemporaryFileCollection that was just uploaded.

        Used by :meth:`.add_success_messages`.
        """
        filenames = [u'"{}"'.format(temporaryfile.filename) for temporaryfile in temporaryfilecollection.files.all()]
        return _(u'Uploaded %(what)s.') % {
            'what': u','.join(filenames)
        }

    def add_success_messages(self, temporaryfilecollection):
        """
        Add success messages on successful upload.

        The ``temporaryfilecollection`` is the TemporaryFileCollection that was just uploaded.

        Defaults to add :meth:`.get_success_message` as a django messages
        success message if :meth:`.get_success_message` returns anything.

        You can override this to add multiple messages or to show messages in some other way.
        """
        success_message = self.get_success_message(temporaryfilecollection)
        if success_message:
            messages.success(self.request, success_message)

    def form_valid(self, form):
        collectionid = form.cleaned_data['filecollectionid']
        try:
            temporaryfilecollection = self.get_collectionqueryset().get(id=collectionid)
        except TemporaryFileCollection.DoesNotExist:
            return http.HttpResponseNotFound()
        else:
            self.upload_files_to_archive(temporaryfilecollection)
            self.add_success_messages(temporaryfilecollection)
            temporaryfilecollection.clear_files_and_delete()
            return http.HttpResponseRedirect(self.get_success_url())


class App(crapp.App):
    appurls = [
        crapp.Url(
            r'^$',
            ArchiveImagesListView.as_view(),
            name=crapp.INDEXVIEW_NAME),
        crapp.Url(
            r'^singleselect$',
            ArchiveImagesSingleSelectView.as_view(),
            name='singleselect'),
        crapp.Url(
            r'^create$',
            ArchiveImageCreateView.as_view(),
            name="create"),
        crapp.Url(
            r'^edit/(?P<pk>\d+)$',
            ArchiveImageUpdateView.as_view(),
            name="edit"),
        crapp.Url(
            r'^delete/(?P<pk>\d+)$',
            ArchiveImageDeleteView.as_view(),
            name="delete"),
        crapp.Url(
            r'bulkadd$',
            ArchiveImageBulkAddView.as_view(),
            name='bulkadd'
        )
    ]



from __future__ import unicode_literals
from django.utils.translation import ugettext_lazy as _
from django import forms
from django.views.generic import TemplateView
from django.views.generic.edit import FormMixin
from django.shortcuts import render
from django import http
from crispy_forms import layout

from django_cradmin import crapp
from django_cradmin.crispylayouts import CradminFormHelper


class MultiSelectView(TemplateView):
    """
    Base view for multiselect views (views used when the
    user selects one or more items for some action).

    You must override:
    - :meth:`.get_queryset_for_role`.
    - :meth:`.object_selection_valid`
    """

    #: The view name for the success page.
    #: You can set this, or implement :meth:`.get_listing_url`.
    #: Defaults to :obj:`django_cradmin.crapp.INDEXVIEW_NAME`.
    success_viewname = crapp.INDEXVIEW_NAME

    def get_queryset_for_role(self, role):
        """
        Get a queryset with all objects of :obj:`.model`  that
        the current role can access.
        """
        raise NotImplementedError()

    def get_queryset(self):
        """
        DO NOT override this. Override :meth:`.get_queryset_for_role`
        instead.
        """
        queryset = self.get_queryset_for_role(self.request.cradmin_role)
        return queryset

    def _get_selected_objects_form_class(self):
        class MultiSelectForm(forms.Form):
            selected_objects = forms.ModelMultipleChoiceField(
                widget=forms.MultipleHiddenInput,
                queryset=self.get_queryset())
        return MultiSelectForm

    def _get_selected_objects_form(self):
        form_class = self._get_selected_objects_form_class()
        return form_class(data=self.request.POST)

    def post(self, request, *args, **kwargs):
        """
        Handles POST requests, instantiating a form instance with the passed
        POST variables and then checked for validity.
        """
        self.selected_objects_form = self._get_selected_objects_form()
        if self.selected_objects_form.is_valid():
            self.selected_objects = self.selected_objects_form.cleaned_data['selected_objects']
            return self.object_selection_valid()
        else:
            return self.object_selection_invalid(self.selected_objects_form)

    def is_first_load(self):
        """
        Can be used by subclasses to check if this is the
        first/initial load of this view (right after
        the user submits the form where they selected what
        to perform the action for).
        """
        return 'is_the_multiselect_form' in self.request.POST

    def object_selection_valid(self):
        """
        Called if the form used to validate the selected objects is valid.
        Must be overridden.

        The selected objects is available as ``self.selected_objects``.
        """
        raise NotImplementedError()

    def object_selection_invalid(self, form):
        """
        Called if the form used to validate the selected objects is invalid.
        Must be overridden.
        """
        return render(self.request, 'django_cradmin/error.django.html', {
            'error': _(
                'Invalid selection. This is usually caused by someone else changing '
                'permissions while you where selecting items to edit.')
        })

    def get_context_data(self, **kwargs):
        context = super(MultiSelectView, self).get_context_data(**kwargs)
        context['selected_objects_form'] = self.selected_objects_form
        context['selected_objects'] = self.selected_objects
        return context

    def get_success_url(self):
        """
        Get the URL to redirect to on success.

        Defaults to :obj:`.success_viewname`.
        """
        return self.request.cradmin_app.reverse_appurl(self.success_viewname)

    def success_redirect_response(self):
        """
        Return a :class:`django.http.HttpResponseRedirect` response with
        :meth:`.get_success_url` as the URL.
        """
        return http.HttpResponseRedirect(self.get_success_url())


class NoFormActionFormHelper(CradminFormHelper):
    form_action = ''


class MultiSelectFormView(MultiSelectView, FormMixin):
    """
    Extends :class:`.MultiSelectView` and :class:`django.views.generic.edit.FormMixin`
    to make a base class for multiselect form views.

    You must override:
    - :meth:`.get_queryset_for_role`
    - :meth:`.form_valid`
    - :meth:`.get_field_layout`

    And you have to set:
    - :obj:`.model`, or override a method as explained in its docs.
    - ``form_class`` or override ``get_form_class(...)`` as documented in the Django FormMixin docs.

    You will most likely also want to override :meth:.get_buttons`.
    """
    template_name = 'django_cradmin/viewhelpers/multiselect/formview.django.html'

    #: The model class to edit. You do not have to specify
    #: this, but if you do not specify this, you have to override
    #: :meth:`.get_pagetitle`.
    model = None

    #: The field that should always be set to the current role.
    #: Adds a hidden field with the correct value for the current
    #: role. See :meth:`.get_hidden_fields`.
    roleid_field = None

    def get_form_kwargs(self):
        """
        Returns the keyword arguments for instantiating the form.

        Overridden to handle the fact that we always POST. So instead
        of distinquishing between POST and GET, we check
        :meth:`.MultiSelectView.is_first_load`.
        """
        kwargs = {
            'initial': self.get_initial(),
            'prefix': self.get_prefix(),
        }

        if not self.is_first_load() and self.request.method in ('POST', 'PUT'):
            kwargs.update({
                'data': self.request.POST,
                'files': self.request.FILES,
            })
        return kwargs

    def object_selection_valid(self):
        form_class = self.get_form_class()
        form = self.get_form(form_class)
        if self.is_first_load():
            return self.first_load(form)
        else:
            if form.is_valid():
                return self.form_valid(form)
            else:
                return self.form_invalid(form)

    def first_load(self, form):
        """
        Called on the first load of the page (right after
        the user submits the form where they selected what
        to edit).
        """
        return self.render_to_response(self.get_context_data(form=form))

    def form_invalid(self, form):
        """
        Called when the form is invalid.
        """
        return self.render_to_response(self.get_context_data(form=form))

    def form_valid(self, form):
        """
        Called when the form is valid. Must be overridden.

        You will normally want to do something with the form data
        here, and redirect to a success page.
        """
        raise NotImplementedError()

    def get_pagetitle(self):
        """
        Get the page title/heading.

        Defaults to the ``verbose_name_plural`` of the :obj:`.model`.
        """
        return self.model._meta.verbose_name_plural

    def get_context_data(self, **kwargs):
        context = super(MultiSelectFormView, self).get_context_data(**kwargs)
        context['pagetitle'] = self.get_pagetitle()
        context['formhelper'] = self.get_formhelper()
        return context

    def get_field_layout(self):
        """
        Get a list/tuple of fields. These are added to a ``crispy_forms.layout.Layout``.
        Defaults to a all fields on the model. If :obj:`.fields`, we use those fields.

        Simple example (same as specifying the fields in :obj:`.fields`)::

            from crispy_forms import layout

            class MyCreateView(CreateView):
                def get_field_layout(self):
                    return [
                        layout.Div(
                            'title', 'name', 'size', 'tags',
                            css_class='cradmin-globalfields')
                    ]

        A slightly more complex example::

            from crispy_forms import layout

            class MyEditView(MultiSelectFormView):
                def get_field_layout(self):
                    return [
                        layout.Div('data', css_class="cradmin-focusfield"),
                    ]

        """
        raise NotImplementedError()

    def get_hidden_fields(self):
        """
        Get hidden fields for the form.

        If you set :obj:`.roleid_field`, a hidden field named whatever you
        specify in :obj:`.roleid_field` with value set to the current role ID
        is added automatically.

        Returns:
            An iterable of :class:`crispy_forms.layout.Hidden` objects.
        """
        fields = []
        if self.roleid_field:
            roleid = self.request.cradmin_instance.get_roleid(self.request.cradmin_role)
            fields.append(layout.Hidden(self.roleid_field, roleid))
        return fields

    def get_buttons(self):
        """
        Get buttons for the form, normally one or more submit button.

        Each button must be a crispy form layout object, typically some
        subclass of :class:`crispy_forms.layout.Submit`.

        See:
            This method is used by :meth:`.get_button_layout`.
        """
        return []

    def get_button_layout(self):
        """
        Get the button layout. This is added to the crispy form layout.

        Defaults to a :class:`crispy_forms.layout.Div` with css class
        ``django_cradmin_submitrow`` containing all the buttons
        returned by :meth:`.get_buttons`.
        """
        return [
            layout.Div(*self.get_buttons(), css_class="django_cradmin_submitrow")
        ]

    def get_formhelper(self):
        """
        Get a :class:`crispy_forms.helper.FormHelper`.

        You normally do not need to override this directly. Instead
        you should override:

        - :meth:`.get_field_layout`.
        - :meth:`.get_hidden_fields`
        - :meth:`.get_buttons` (or perhaps :meth:`.get_button_layout`)
        """
        helper = NoFormActionFormHelper()
        layoutargs = list(self.get_field_layout()) + list(self.get_button_layout()) + list(self.get_hidden_fields())
        helper.layout = layout.Layout(*layoutargs)
        helper.form_tag = False  # NOTE: We render the form tag in the template because we render multiple forms.
        helper.disable_csrf = True  # NOTE: We render csrf token in the template.
        return helper



from __future__ import unicode_literals

from crispy_forms import layout
from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.urlresolvers import reverse
from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext_lazy as _
from django import forms
from django.views.generic import FormView

from crispy_forms.helper import FormHelper

from django_cradmin.apps.cradmin_email import emailutils
from django_cradmin.apps.cradmin_generic_token_with_metadata.models import GenericTokenWithMetadata, \
    get_expiration_datetime_for_app
from django_cradmin.crispylayouts import PrimarySubmitLg


class PasswordResetEmail(emailutils.AbstractEmail):
    subject_template = 'cradmin_resetpassword/email/subject.django.txt'
    html_message_template = 'cradmin_resetpassword/email/html_message.django.html'

    def get_context_data(self):
        context = super(PasswordResetEmail, self).get_context_data()
        context.update({
            'DJANGO_CRADMIN_SITENAME': settings.DJANGO_CRADMIN_SITENAME
        })
        return context


class EmailForm(forms.Form):
    email = forms.EmailField()

    def clean_email(self):
        email = self.cleaned_data['email']
        user_model = get_user_model()
        if not user_model.objects.filter(email=email).exists():
            raise forms.ValidationError("No account with this email address found")
        return email


class BeginPasswordResetView(FormView):
    template_name = 'cradmin_resetpassword/begin.django.html'
    form_class = EmailForm

    def get_formhelper(self):
        helper = FormHelper()
        helper.form_action = '#'
        helper.form_id = 'django_cradmin_resetpassword_begin_form'
        helper.form_show_labels = False
        helper.layout = layout.Layout(
            layout.Field('email', css_class='input-lg', placeholder=_('Email'), focusonme='focusonme'),
            PrimarySubmitLg('submit', _('Search'))
        )
        return helper

    def get_context_data(self, **kwargs):
        context = super(BeginPasswordResetView, self).get_context_data(**kwargs)
        context['formhelper'] = self.get_formhelper()
        return context

    def get_success_url(self):
        return reverse('cradmin-resetpassword-email-sent')

    def __send_email(self, user, reset_url):
        PasswordResetEmail(
            recipient=user.email,
            from_email=getattr(settings, 'DJANGO_CRADMIN_RESETPASSWORD_FROM_EMAIL', settings.DEFAULT_FROM_EMAIL),
            extra_context_data={
                'user': user,
                'reset_url': reset_url
            }
        ).send()

    def _generate_token(self, user):
        return GenericTokenWithMetadata.objects.generate(
            app='cradmin_resetpassword',
            content_object=user,
            expiration_datetime=get_expiration_datetime_for_app('cradmin_resetpassword')
        ).token

    def __generate_reset_url(self, user):
        reset_url = reverse('cradmin-resetpassword-reset', kwargs={
            'token': self._generate_token(user)
        })
        return self.request.build_absolute_uri(reset_url)

    def form_valid(self, form):
        email = form.cleaned_data['email']
        user_model = get_user_model()
        user = get_object_or_404(user_model, email=email)
        reset_url = self.__generate_reset_url(user=user)
        self.__send_email(user=user, reset_url=reset_url)
        return super(BeginPasswordResetView, self).form_valid(form)



from __future__ import unicode_literals
from functools import wraps

from django.conf import settings
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.core.exceptions import ObjectDoesNotExist
from django.http import HttpResponse
from django.utils.encoding import force_str
from django.shortcuts import resolve_url

from django_cradmin.registry import cradmin_instance_registry


def cradminview(view_function, redirect_field_name=REDIRECT_FIELD_NAME, login_url=None):
    """
    Decorator for django_cradmin views.

    Protects the view, making it impossible to access unless the requesting user
    has the role defined by the named url variable ``roleid``.

    Adds the following variables to the request:

        - ``cradmin_instance``: The detected cradmin instance.
        - ``cradmin_role``: The detected cradmin role.
    """

    @wraps(view_function)
    def wrapper(request, *args, **kwargs):
        if request.user.is_authenticated():
            roleid = kwargs.pop('roleid')
            cradmin_instance = cradmin_instance_registry.get_current_instance(request)
            role = cradmin_instance.get_role_from_roleid(roleid)
            if not role:
                response = cradmin_instance.invalid_roleid_response(roleid)
            try:
                role_from_rolequeryset = cradmin_instance.get_role_from_rolequeryset(role)
            except ObjectDoesNotExist:
                response = cradmin_instance.missing_role_response(role)
            else:
                request.cradmin_instance = cradmin_instance
                request.cradmin_role = role_from_rolequeryset
                response = view_function(request, *args, **kwargs)

            if isinstance(response, HttpResponse):
                http_headers = cradmin_instance.get_common_http_headers()
                if http_headers:
                    for headerattribute, headervalue in http_headers.items():
                        response[headerattribute] = headervalue
            return response
        else:
            # Redirect to login just like login_required()
            from django.contrib.auth.views import redirect_to_login
            path = request.build_absolute_uri()
            resolved_login_url = force_str(
                resolve_url(login_url or settings.LOGIN_URL))
            return redirect_to_login(path, resolved_login_url, redirect_field_name)

    return wrapper



from __future__ import unicode_literals
from builtins import str
from builtins import range
import htmls
import mock
from django.test import TestCase, RequestFactory

from django_cradmin.tests.viewhelpers.cradmin_viewhelpers_testapp.models import TestModel
from django_cradmin.viewhelpers import objecttable
from django_cradmin.django_cradmin_testapp import models as testmodels


class TestColumn(TestCase):
    def setUp(self):
        class TestColSubclass(objecttable.Column):
            modelfield = 'testfield'

            def __init__(self, **kwargs):
                super(TestColSubclass, self).__init__(**kwargs)

        self.model_testobject = TestModel()
        view = mock.MagicMock()
        view.model = self.model_testobject
        self.column_subclass = TestColSubclass(view=view, columnindex=0)

    def test_get_header_not_implemented(self):
        test = objecttable.Column(view=None, columnindex=0)

        with self.assertRaises(NotImplementedError):
            test.get_header()

    def test_get_header(self):
        self.assertEqual("Test Value", self.column_subclass.get_header())

    def test_render_value(self):
        self.model_testobject.testfield = u'test_value'
        self.assertEqual("test_value", self.column_subclass.render_value(self.model_testobject))

    # check that you get an exception when running render_value without having modelfield..
    def test_render_value_not_implemented(self):
        col = objecttable.Column(view=None, columnindex=0)

        with self.assertRaises(NotImplementedError):
            col.render_value(None)

    # check that you get an exception when running render_cell_content without overriding with subclass..
    def test_render_cell_not_implemented(self):
        col = objecttable.Column(view=None, columnindex=0)

        with self.assertRaises(NotImplementedError):
            col.render_cell_content(None)


class TestPlainTextColumn(TestCase):
    def setUp(self):
        class TestColSubclass(objecttable.PlainTextColumn):
            modelfield = 'testfield'

            def __init__(self, **kwargs):
                super(TestColSubclass, self).__init__(**kwargs)

        self.model_testobject = TestModel()
        view = mock.MagicMock()
        view.model = self.model_testobject
        self.column_subclass = TestColSubclass(view=view, columnindex=0)

    def test_render_cell(self):
        self.model_testobject.testfield = 'test_value'
        self.assertEquals(
            '<span class="objecttable-cellvalue">test_value</span>',
            self.column_subclass.render_cell_content(self.model_testobject).strip())


class TestSingleActionColumn(TestCase):
    def setUp(self):
        class TestIncompleteColSubclass(objecttable.SingleActionColumn):
            modelfield = 'testfield'

            def __init__(self, **kwargs):
                super(TestIncompleteColSubclass, self).__init__(**kwargs)

        class TestColSubclass(objecttable.SingleActionColumn):
            modelfield = 'testfield'

            def __init__(self, **kwargs):
                super(TestColSubclass, self).__init__(**kwargs)

            def get_actionurl(self, obj):
                return 'www.example.com/{}'.format(obj.testfield)

        self.model_testobject = TestModel(testfield="test_value")
        view = mock.MagicMock()
        view.model = self.model_testobject
        self.column_subclass = TestColSubclass(view=view, columnindex=0)
        self.column_subclass_incomplete = TestIncompleteColSubclass(view=view, columnindex=0)

    def test_get_actionurl_ExceptionRaised(self):
        with self.assertRaises(NotImplementedError):
            self.column_subclass_incomplete.get_actionurl(self.model_testobject)

    def test_render_cell_ExceptionRaised(self):
        with self.assertRaises(NotImplementedError):
            self.column_subclass_incomplete.render_cell_content(self.model_testobject)

    def test_render_cell(self):
        expected = '<a href="www.example.com/test_value" class="objecttable-cellvalue-link">test_value</a>'
        self.assertEquals(self.column_subclass.render_cell_content(self.model_testobject), expected)


class TestMultiActionColumn(TestCase):
    def setUp(self):
        class TestIncompleteColSubclass(objecttable.MultiActionColumn):
            modelfield = 'testfield'

            def __init__(self, **kwargs):
                super(TestIncompleteColSubclass, self).__init__(**kwargs)

        class TestColSubclass(objecttable.MultiActionColumn):
            modelfield = 'testfield'

            def __init__(self, **kwargs):
                super(TestColSubclass, self).__init__(**kwargs)

            def get_buttons(self, obj):
                return [objecttable.Button(label="Btn1", url="www.example.com/btn1"),
                        objecttable.Button(label="Btn2", url="www.example.com/btn2")]

        self.model_testobject = TestModel(testfield="test_value")
        view = mock.MagicMock()
        view.model = self.model_testobject
        self.column_subclass = TestColSubclass(view=view, columnindex=0)
        self.column_subclass_incomplete = TestIncompleteColSubclass(view=view, columnindex=0)

    def test_get_buttons_ExceptionRaised(self):
        with self.assertRaises(NotImplementedError):
            self.column_subclass_incomplete.get_buttons(self.model_testobject)

    def test_render_cell_ExceptionRaised(self):
        with self.assertRaises(NotImplementedError):
            self.column_subclass_incomplete.render_cell_content(self.model_testobject)

    def test_render_cell(self):
        result = self.column_subclass.render_cell_content(self.model_testobject)
        selector = htmls.S(result)
        self.assertEqual(selector.one('p.objecttable-cellvalue').alltext_normalized,
                         'test_value')
        self.assertTrue(selector.exists('p.objecttable-cellbuttons'))
        self.assertEqual(selector.count('a'), 2)


class TestButton(TestCase):
    def test_render_with_icon_and_class(self):
        btn = objecttable.Button(
            label="My Btn", url="www.example.com/mybtnurl",
            buttonclass="btn btn-danger btn-sm",
            icon="glyphicon glyphicon-shopping-cart")
        selector = htmls.S(btn.render())
        self.assertEqual(selector.one('a')['href'], 'www.example.com/mybtnurl')
        self.assertEqual(selector.one('a')['class'], 'btn btn-danger btn-sm')
        self.assertEqual(selector.one('a').alltext_normalized, 'My Btn')
        self.assertEqual(selector.one('a span')['class'], 'glyphicon glyphicon-shopping-cart')

    def test_render_simple(self):
        btn = objecttable.Button(label="My Btn", url="www.example.com/mybtnurl")
        selector = htmls.S(btn.render())
        self.assertEqual(selector.one('a')['href'], 'www.example.com/mybtnurl')
        self.assertEqual(selector.one('a')['class'], 'btn btn-default btn-sm')
        self.assertEqual(selector.one('a').alltext_normalized, 'My Btn')


class TestOrderingStringParser(TestCase):
    def test_parse_empty(self):
        orderingqueryarg = objecttable.OrderingStringParser('')
        self.assertEquals(len(orderingqueryarg.orderingdict), 0)

    def test_parse_single(self):
        orderingqueryarg = objecttable.OrderingStringParser('a3')
        self.assertEquals(len(orderingqueryarg.orderingdict), 1)
        self.assertTrue(orderingqueryarg.orderingdict[3])

    def test_parse_multi(self):
        orderingqueryarg = objecttable.OrderingStringParser('a3.d1')
        self.assertEquals(len(orderingqueryarg.orderingdict), 2)
        self.assertTrue(orderingqueryarg.orderingdict[3].order_ascending)
        self.assertFalse(orderingqueryarg.orderingdict[1].order_ascending)

    def test_remove_column(self):
        self.assertEqual(
            objecttable.OrderingStringParser('a3.d1.a6').remove_column(6),
            'a3.d1')
        self.assertEqual(
            objecttable.OrderingStringParser('a3.d1.a6').remove_column(1),
            'a3.a6')

    def test_remove_nonexisting_colum(self):
        self.assertEqual(
            objecttable.OrderingStringParser('').remove_column(2),
            '')
        self.assertEqual(
            objecttable.OrderingStringParser('a1').remove_column(2),
            'a1')
        self.assertEqual(
            objecttable.OrderingStringParser('a3.d1.a6').remove_column(2),
            'a3.d1.a6')

    def test_flip_existing_column(self):
        self.assertEqual(
            objecttable.OrderingStringParser('d1').flip_column(1),
            'a1')
        self.assertEqual(
            objecttable.OrderingStringParser('a3.d1.a6').flip_column(1),
            'a3.a1.a6')
        self.assertEqual(
            objecttable.OrderingStringParser('a3.d1.a6').flip_column(3),
            'd3.d1.a6')

    def test_flip_new_column(self):
        self.assertEqual(
            objecttable.OrderingStringParser('').flip_column(1),
            'a1')
        self.assertEqual(
            objecttable.OrderingStringParser('a3').flip_column(1),
            'a3.a1')


class TestObjectTableView(TestCase):
    def setUp(self):
        self.factory = RequestFactory()

    def _mock_request(self, request):
        request.cradmin_role = mock.MagicMock()
        request.cradmin_app = mock.MagicMock()

    def test_empty(self):
        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.none()

        request = self.factory.get('/test')
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)
        # selector.one('#django_cradmin_contentwrapper').prettyprint()
        self.assertFalse(selector.exists('#objecttableview-table'))
        self.assertEquals(
            selector.one('#objecttableview-no-items-message').alltext_normalized,
            'No some items')

    def test_empty_hide_search(self):
        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            searchfields = ['name']

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.none()

        request = self.factory.get('/test')
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)
        self.assertFalse(selector.exists('.cradmin-searchform'))

    def test_paginate_by_singlepage(self):
        testmodels.SomeItem.objects.bulk_create(
            [testmodels.SomeItem(name=str(x)) for x in range(4)])

        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            paginate_by = 4

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.all()

        request = self.factory.get('/test')
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)
        # selector.one('#django_cradmin_contentwrapper').prettyprint()
        self.assertEqual(selector.count('#objecttableview-table>tbody>tr'), 4)
        self.assertFalse(selector.exists('#django_cradmin_contentwrapper .pager'))

    def test_paginate_by_firstpage(self):
        testmodels.SomeItem.objects.bulk_create(
            [testmodels.SomeItem(name=str(x)) for x in range(5)])

        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            paginate_by = 4

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.all()

        request = self.factory.get('/test')
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)
        self.assertEqual(selector.count('#objecttableview-table>tbody>tr'), 4)
        self.assertTrue(selector.exists('#django_cradmin_contentwrapper .pager'))
        self.assertTrue(selector.exists('#django_cradmin_contentwrapper .pager .previous.disabled'))
        self.assertTrue(selector.exists('#django_cradmin_contentwrapper .pager .next'))
        self.assertFalse(selector.exists('#django_cradmin_contentwrapper .pager .next.disabled'))

    def test_paginate_by_lastpage(self):
        testmodels.SomeItem.objects.bulk_create(
            [testmodels.SomeItem(name=str(x)) for x in range(5)])

        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            paginate_by = 4

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.all()

        request = self.factory.get('/test', {
            'page': 2
        })
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)
        self.assertEqual(selector.count('#objecttableview-table>tbody>tr'), 1)
        self.assertTrue(selector.exists('#django_cradmin_contentwrapper .pager'))
        self.assertTrue(selector.exists('#django_cradmin_contentwrapper .pager .previous'))
        self.assertFalse(selector.exists('#django_cradmin_contentwrapper .pager .previous.disabled'))
        self.assertTrue(selector.exists('#django_cradmin_contentwrapper .pager .next.disabled'))

    def test_paginate_by_middlepage(self):
        testmodels.SomeItem.objects.bulk_create(
            [testmodels.SomeItem(name=str(x)) for x in range(9)])

        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            paginate_by = 4

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.all()

        request = self.factory.get('/test', {
            'page': 2
        })
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)
        # selector.one('#django_cradmin_contentwrapper').prettyprint()
        self.assertEqual(selector.count('#objecttableview-table>tbody>tr'), 4)
        self.assertTrue(selector.exists('#django_cradmin_contentwrapper .pager'))
        self.assertTrue(selector.exists('#django_cradmin_contentwrapper .pager .previous'))
        self.assertTrue(selector.exists('#django_cradmin_contentwrapper .pager .next'))
        self.assertFalse(selector.exists('#django_cradmin_contentwrapper .pager .previous.disabled'))
        self.assertFalse(selector.exists('#django_cradmin_contentwrapper .pager .next.disabled'))

    def test_render_single_simple_column(self):
        testmodels.SomeItem.objects.create(name='Item One')

        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            columns = ['name']

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.all()

        request = self.factory.get('/test')
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)

        self.assertEquals(selector.count('#objecttableview-table>thead>tr>th'), 1)
        self.assertEquals(
            selector.one('#objecttableview-table>thead>tr>th').alltext_normalized,
            'The name - Ordered descending - Click to order ascending')

        self.assertEqual(selector.count('#objecttableview-table>tbody>tr'), 1)
        self.assertEquals(selector.count('#objecttableview-table>tbody>tr>td'), 1)
        self.assertEquals(
            selector.one('#objecttableview-table>tbody>tr>td').alltext_normalized,
            'Item One')

    def test_render_multiple_simple_columns(self):
        testmodels.SomeItem.objects.create(name='Item One', somenumber=10)

        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            columns = ['name', 'somenumber']

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.all()

        request = self.factory.get('/test')
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)

        self.assertEquals(selector.count('#objecttableview-table>thead>tr>th'), 2)
        self.assertEquals(
            selector.one('#objecttableview-table>thead>tr>th:first-child').alltext_normalized,
            'The name - Ordered descending - Click to order ascending')
        self.assertEquals(
            selector.one('#objecttableview-table>thead>tr>th:last-child').alltext_normalized,
            'Somenumber - Ordered descending - Click to order ascending')

        self.assertEqual(selector.count('#objecttableview-table>tbody>tr'), 1)
        self.assertEquals(selector.count('#objecttableview-table>tbody>tr>td'), 2)
        self.assertEquals(
            selector.one('#objecttableview-table>tbody>tr>td:first-child').alltext_normalized,
            'Item One')
        self.assertEquals(
            selector.one('#objecttableview-table>tbody>tr>td:last-child').alltext_normalized,
            '10')

    def test_render_order_ascending_singlecolumn(self):
        testmodels.SomeItem.objects.create(name='Item A')
        testmodels.SomeItem.objects.create(name='Item B')
        testmodels.SomeItem.objects.create(name='Item C')

        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            columns = ['name']

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.all()

        request = self.factory.get('/test', {
            'ordering': 'a0'
        })
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)

        self.assertEqual(selector.count('#objecttableview-table>tbody>tr'), 3)
        self.assertEquals(
            selector.one('#objecttableview-table>tbody>tr:first-child>td').alltext_normalized,
            'Item A')
        self.assertEquals(
            selector.one('#objecttableview-table>tbody>tr:last-child>td').alltext_normalized,
            'Item C')

        self.assertEquals(
            selector.one('#objecttableview-table>thead>tr>th').alltext_normalized,
            'The name - Ordered ascending - Click to order descending')

    def test_render_order_descending_column(self):
        testmodels.SomeItem.objects.create(name='Item A')
        testmodels.SomeItem.objects.create(name='Item B')
        testmodels.SomeItem.objects.create(name='Item C')

        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            columns = ['name']

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.all()

        request = self.factory.get('/test', {
            'ordering': 'd0'
        })
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)
        # selector.one('#django_cradmin_contentwrapper thead').prettyprint()

        self.assertEqual(selector.count('#objecttableview-table>tbody>tr'), 3)
        self.assertEquals(
            selector.one('#objecttableview-table>tbody>tr:first-child>td').alltext_normalized,
            'Item C')
        self.assertEquals(
            selector.one('#objecttableview-table>tbody>tr:last-child>td').alltext_normalized,
            'Item A')

        self.assertEquals(
            selector.one('#objecttableview-table>thead>tr>th').alltext_normalized,
            'The name - Ordered descending - Click to order ascending')

    def test_render_order_multicolumn(self):
        testmodels.SomeItem.objects.create(name='Item A', somenumber=1)
        testmodels.SomeItem.objects.create(name='Item B', somenumber=2)
        testmodels.SomeItem.objects.create(name='Item C', somenumber=2)

        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            columns = ['name', 'somenumber']

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.all()

        request = self.factory.get('/test', {
            'ordering': 'a1.d0'
        })
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)
        # selector.one('#django_cradmin_contentwrapper thead').prettyprint()

        self.assertEquals(selector.count('#objecttableview-table>thead>tr>th'), 2),
        self.assertEquals(
            selector.one('#objecttableview-table>thead>tr>th:first-child').alltext_normalized,
            'The name - Ordered descending - Click to order ascending - Ordering priority 2')
        self.assertEquals(
            selector.one('#objecttableview-table>thead>tr>th:last-child').alltext_normalized,
            'Somenumber - Ordered ascending - Click to order descending - Ordering priority 1')

        self.assertEqual(selector.count('#objecttableview-table>tbody>tr'), 3)
        self.assertEquals(
            selector.one('#objecttableview-table>tbody>tr:first-child>td:first-child').alltext_normalized,
            'Item A')
        self.assertEquals(
            selector.one('#objecttableview-table>tbody>tr:last-child>td:first-child').alltext_normalized,
            'Item B')

    def test_render_search_nomatch(self):
        testmodels.SomeItem.objects.create(name='Item One')

        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            columns = ['name']
            searchfields = ['name']

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.all()

        request = self.factory.get('/test', {
            'search': 'Nothing matches this'
        })
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)
        self.assertEqual(selector.count('#objecttableview-table>tbody>tr'), 0)

    def test_render_search_match(self):
        testmodels.SomeItem.objects.create(name='Item One')
        testmodels.SomeItem.objects.create(name='Item Two')
        testmodels.SomeItem.objects.create(name='Item Three')

        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            columns = ['name']
            searchfields = ['name']

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.all()

        request = self.factory.get('/test', {
            'search': 'Item Two'
        })
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)

        self.assertEqual(selector.count('#objecttableview-table>tbody>tr'), 1)
        self.assertEquals(
            selector.one('#objecttableview-table>tbody>tr>td').alltext_normalized,
            'Item Two')

    def test_show_column_headers(self):
        testmodels.SomeItem.objects.create(name='Item One')

        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            columns = ['name']

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.all()

        request = self.factory.get('/')
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)

        self.assertTrue(selector.exists('#objecttableview-table>thead'))
        self.assertFalse(selector.exists('#objecttableview-table>thead.sr-only'))

    def test_hide_column_headers(self):
        testmodels.SomeItem.objects.create(name='Item One')

        class MyObjectTableView(objecttable.ObjectTableView):
            model = testmodels.SomeItem
            columns = ['name']
            hide_column_headers = True

            def get_queryset_for_role(self, role):
                return testmodels.SomeItem.objects.all()

        request = self.factory.get('/')
        self._mock_request(request)
        response = MyObjectTableView.as_view()(request)
        response.render()
        selector = htmls.S(response.content)

        self.assertTrue(selector.exists('#objecttableview-table>thead'))
        self.assertTrue(selector.exists('#objecttableview-table>thead.sr-only'))



from __future__ import unicode_literals
from django.db import models


class TstRole(models.Model):
    """
    Used by tests that require a role object.
    """


class SomeItem(models.Model):
    name = models.CharField(
        max_length=30,
        verbose_name='The name'
    )
    somenumber = models.IntegerField(null=True, blank=True)
    text = models.TextField(
        blank=True, null=False, default=''
    )



from __future__ import unicode_literals
from django.test import TestCase
from django_cradmin.apps.cradmin_register_account.forms.email import EmailUserCreateAccountForm
from django_cradmin.apps.cradmin_register_account.tests.cradmin_register_account_testapp.models import EmailUser


class EmailUserCreateAccountFormStaticUserModel(EmailUserCreateAccountForm):
    class Meta(EmailUserCreateAccountForm.Meta):
        model = EmailUser


class TestEmailUserCreateAccountForm(TestCase):
    def test_is_valid(self):
        form = EmailUserCreateAccountFormStaticUserModel({
            'password1': 'test',
            'password2': 'test',
            'email': 'test@example.com'
        })
        self.assertTrue(form.is_valid())

    def test_save(self):
        form = EmailUserCreateAccountFormStaticUserModel({
            'password1': 'test',
            'password2': 'test',
            'email': 'test@example.com'
        })
        form.is_valid()
        self.assertEquals(EmailUser.objects.count(), 0)
        form.save()
        self.assertEquals(EmailUser.objects.count(), 1)
        created_user = EmailUser.objects.first()
        self.assertEqual(created_user.email, 'test@example.com')
        self.assertTrue(created_user.has_usable_password())
        self.assertTrue(created_user.check_password('test'))

    def test_passwords_do_not_match(self):
        form = EmailUserCreateAccountFormStaticUserModel({
            'password1': 'test1',
            'password2': 'test2',
            'email': 'unused@example.com'
        })
        self.assertFalse(form.is_valid())
        self.assertEquals(form.non_field_errors(), [u'The passwords do not match.'])

    def test_email_is_not_unique(self):
        EmailUser.objects.create(email='test@example.com')
        form = EmailUserCreateAccountFormStaticUserModel({
            'password1': 'unused',
            'password2': 'unused',
            'email': 'test@example.com'
        })
        self.assertFalse(form.is_valid())
        self.assertEquals(form.errors, {
            'email': [u'Email user with this Email already exists.']
        })



from __future__ import unicode_literals
from django.conf.urls import patterns, url
from django_cradmin.apps.cradmin_authenticate.views.login import LoginView


urlpatterns = patterns(
    'django_cradmin.apps.cradmin_authenticate.views',
    url(r'^login$', LoginView.as_view(), name='cradmin-authenticate-login'),
    url(r'^logout$', 'logout.cradmin_logoutview', name='cradmin-authenticate-logout'),
)



from django.http import QueryDict
from future.standard_library import install_aliases

install_aliases()

from urllib.parse import urlsplit, urlunsplit


def create_querydict(querystringargs, initial_query_string=None, ignore_none_values=True):
    """
    Parameters:
        querystringargs (dict): The querystring args to add/replace.
        initial_query_string (str): The initial querystring. Any ovelapping
            keys between this and ``querystringargs`` is overridden by
            the value in ``querystringargs``.
        ignore_none_values (bool): If this is ``True`` (default),
            we ignore ``None`` values in ``querystringargs``.

    Returns:
        The created :class:`django.http.request.QueryDict`.
    """
    querydict = QueryDict(query_string=initial_query_string, mutable=True)
    for key, value in querystringargs.items():
        if ignore_none_values and value is None:
            continue
        querydict[key] = value
    return querydict


def update_querystring(url, querystringargs, ignore_none_values=True):
    """
    Update the querystring portion of the given ``url``.

    Parameters:
        querystringargs (dict): The querystring args to add/replace.
        ignore_none_values (bool): If this is ``True`` (default),
            we ignore ``None`` values in ``querystringargs``.

    Returns:
        The updated url.

    Examples:

        Add querystring argument::

            from django_cradmin import urlutils
            urlutils.update_querystring('http://example.com', {'search': 'test'})

        Update querystring argument::

            urlutils.update_querystring('http://example.com?search=something&page=2',
                {'search': 'updated'})
    """
    parsed_url = urlsplit(url)
    querydict = create_querydict(querystringargs=querystringargs,
                                 initial_query_string=parsed_url.query,
                                 ignore_none_values=ignore_none_values)
    return urlunsplit((
        parsed_url.scheme,
        parsed_url.netloc,
        parsed_url.path,
        querydict.urlencode(),
        parsed_url.fragment
    ))
